@@ -1,4 +1,4 @@
-# $Id: JobQueue.pm,v 1.8 2002/12/10 04:47:53 rcaputo Exp $
+# $Id: JobQueue.pm 23 2006-08-09 13:23:57Z rcaputo $
 # License and documentation are after __END__.
 
 package POE::Component::JobQueue;
@@ -6,7 +6,7 @@ package POE::Component::JobQueue;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '0.53';
+$VERSION = '0.55';
 
 use Carp qw (croak);
 
@@ -63,7 +63,7 @@ sub spawn {
     $ack_alias = undef unless defined $ack_alias and length $ack_alias;
 
     my $ack_state = delete $active->{AckState};
-    $ack_state = undef unless defined $ack_alias and length $ack_alias;
+    $ack_state = undef unless defined $ack_state and length $ack_state;
 
     croak "$type must have neither or both AckAlias and AckState"
       if defined($ack_alias) xor defined($ack_state);
@@ -138,6 +138,7 @@ sub poco_jobqueue_active_start {
   # fake an initial poll to get things started.
   $heap->{worker_count}  = 0;
   $heap->{pending_polls} = 0;
+  $heap->{latest_worker} = 0;
 
   # Register an alias.
   $kernel->alias_set($alias);
@@ -193,10 +194,26 @@ sub poco_jobqueue_both_child {
           $heap->{worker_limit}, ")"
         ) if $heap->{worker_count} > $heap->{worker_limit};
     $heap->{worker_count}--;
-    $kernel->yield('dequeue');
+    $kernel->yield('dequeue') unless (
+      $heap->{latest_worker} or $heap->{stopped}
+    );
   }
 }
 
+# Remove the alias, stop active polling and delete outstanding job queue
+
+sub poco_jobqueue_both_stop {
+  my ($kernel, $heap) = @_[KERNEL, HEAP];
+
+  $kernel->alias_remove($heap->{alias});
+  $kernel->alarm_remove_all();
+
+  delete $heap->{pollinterval} if ($heap->{pollinterval});
+  delete $heap->{job_queue} if ($heap->{job_queue});
+
+  $heap->{stopped} = 1;
+}
+
 # Attempt to fill empty worker slots.
 
 # This is a token for ARG0 that signifies this was a timed event.
@@ -221,7 +238,10 @@ sub poco_jobqueue_active_dequeue {
   }
 
   # Attempt to fill the empty worker slots.
-  while ($heap->{worker_count} < $heap->{worker_limit}) {
+  while (
+    not $heap->{stopped}
+    and $heap->{worker_count} < $heap->{worker_limit}
+  ) {
 
     # Call the worker to fetch a new job and spawn a session.
     my $previous_worker_count = $heap->{worker_count};
@@ -230,10 +250,11 @@ sub poco_jobqueue_active_dequeue {
     # If the worker count hasn't changed, then we've run out of jobs.
     # Begin polling, if applicable, and exit the spawn loop.
     if ($heap->{worker_count} == $previous_worker_count) {
-      if (defined $heap->{polling_interval} and !$heap->{pending_polls}) {
+      if (defined $heap->{poll_interval} and !$heap->{pending_polls}) {
         $heap->{pending_polls}++;
-        $kernel->delay( dequeue => $heap->{polling_interval} => TIMED );
+        $kernel->delay( dequeue => $heap->{poll_interval} => TIMED );
       }
+      $heap->{latest_worker}++ unless defined $heap->{poll_interval};
       last;
     }
   }
@@ -245,7 +266,10 @@ sub poco_jobqueue_passive_dequeue {
   my ($kernel, $heap) = @_[KERNEL, HEAP];
 
   # Attempt to fill the empty worker slots.
-  while ($heap->{worker_count} < $heap->{worker_limit}) {
+  while (
+    not $heap->{stopped}
+    and $heap->{worker_count} < $heap->{worker_limit}
+  ) {
 
     # Try to fetch another job from the queue.
     my $next_job = shift @{ $heap->{job_queue} };
@@ -268,6 +292,14 @@ sub poco_jobqueue_passive_enqueue {
   my ($kernel, $sender, $heap, $return_state, @job) =
     @_[KERNEL, SENDER, HEAP, ARG0..$#_];
 
+  if ($heap->{stopped}) {
+    DEBUG and warn(
+      "JQ: $heap->{alias} can not enqueue new jobs after 'stop'\n"
+    );
+
+    return;
+  }
+
   DEBUG and warn "JQ: job queue $heap->{alias} enqueuing a new job";
 
   my $postback = $sender->postback( $return_state, @job );
@@ -371,6 +403,9 @@ POE::Component::JobQueue - a component to manage queues and worker pools
     print "results of finished job: (@job_response)\n";
   }
 
+  # Stop a running queue
+  $kernel->call( 'active' => 'stop' );
+
 =head1 DESCRIPTION
 
 POE::Component::JobQueue manages a finite pool of worker sessions as
@@ -550,6 +585,12 @@ Active JobQueue components act as event generators.  They don't
 receive jobs from the outside; instead, they poll for them and post
 acknowledgements as they're completed.
 
+Running queues can be stopped by posting a "stop" state to the 
+component. Any currently running workers will be allowed to 
+complete, but no new workers will be started.
+
+  $kernel->call( 'queue' => 'stop' ); # Stop the running queue
+
 =head1 SEE ALSO
 
 This component is built upon and POE.  Please see its source code and
